-- We will store the interface manager object here so that we don't have to make multiple GetInterfaceManager calls.
local interfaceMngr = nil;

-- The amazonForm table allows us to store all objects related to the specific form inside the table so that we can easily prevent naming conflicts if we need to add more than one form and track elements from both.
local amazonForm = {};
amazonForm.Form = nil;
amazonForm.RibbonPage = nil;
amazonForm.Browser = nil;
amazonForm.ImportButton = nil;

-- Retrieve our settings and store them in local variables to prevent redundant fetching of settings.  The creation of a table is not necessary, but like the form table above, it makes prevention of naming conflicts easier.
local settings = {}
settings.AutoSearch = GetSetting("AutoSearch");

require "Atlas.AtlasHelpers";

function Init()
	if GetFieldValue("Item", "ItemType") == "MON" then
		interfaceMngr = GetInterfaceManager();
		
		-- Create a form
		amazonForm.Form = interfaceMngr:CreateForm("Amazon", "Script");
		
		-- Add a browser
		amazonForm.Browser = amazonForm.Form:CreateBrowser("Amazon Books", "Amazon Browser", "Amazon Books");
		
		-- Hide the text label
		amazonForm.Browser.TextVisible = false;
		
		-- Since we didn't create a ribbon explicitly before creating our browser, it will have created one using the name we passed the CreateBrowser method.  We can retrieve that one and add our buttons to it.
		amazonForm.RibbonPage = amazonForm.Form:GetRibbonPage("Amazon Books");
		
		-- Create the search and import buttons.  We are storing the import button so we can enable it and disable it as needed.
		amazonForm.RibbonPage:CreateButton("Search", GetClientImage("Search32"), "Search", "Amazon");
		amazonForm.ImportButton = amazonForm.RibbonPage:CreateButton("Import Price", GetClientImage("ImportData32"), "ImportPrice", "Amazon");
		amazonForm.ImportButton.BarButton.Enabled = false;
		
		-- After we add all of our buttons and form elements, we can show the form.
		amazonForm.Form:LoadLayout("AmazonLayout.xml");
		amazonForm.Form:Show();
		
		if settings.AutoSearch then
			Search();
		end
	end
end

function Search()
	local title = GetFieldValue("Item", "Title");

	if title == nil then
		title = "";
	end;

	amazonForm.Browser:RegisterPageHandler("custom", "CheckItemPageLoaded", "ItemPageLoaded", false);
	amazonForm.Browser:Navigate("http://www.amazon.com/s/ref=nb_sb_noss?url=search-alias%3Dstripbooks&field-keywords=" .. AtlasHelpers.UrlEncode(title));	
end

function CheckItemPageLoaded()
	local priceContainer = amazonForm.Browser:GetElementInFrame(nil, "rbbContainer");

	if priceContainer == nil then
		priceContainer = amazonForm.Browser:GetElementInFrame(nil, "buybox");
		if priceContainer == nil then
			priceContainer = amazonForm.Browser:GetElementInFrame(nil, "priceBlock");
			if priceContainer == nil then
				return false;
			end
		end
	end
	
	local priceElements = priceContainer:GetElementsByTagName("span");
	
	if priceElements ~= nil then
		-- We could probably grab and store the element we find here so that we don't have to perform the same search in the ItemPageLoaded method.  We opted not to because, by convention, we prefer to avoid global variables that are only conditionally set.
		for i=0, priceElements.Count - 1 do
			local element = amazonForm.Browser:GetElementByCollectionIndex(priceElements, i);
			if element ~= nil then
				if ClassNamesContain(element:GetAttribute("className"), "offer-price") then
					return true;
				elseif ClassNamesContain(element:GetAttribute("className"), "bb_price") then
					return true;
				end;
			end
		end
	end

	priceElements = priceContainer:GetElementsByTagName("b");
	
	if priceElements == nil then
		return false;
	end
	for i=0, priceElements.Count - 1 do
		local element = amazonForm.Browser:GetElementByCollectionIndex(priceElements, i);
		if element ~= nil then
			if ClassNamesContain(element:GetAttribute("className"), "priceLarge") then
				return true;
			end
		end
	end

	return false;
end

function CheckItemPageLeft()
	if CheckItemPageLoaded() then
		return false;
	else
		return true;
	end;
end

-- We use the ItemPageLeft and ItemPageLoaded handlers to provide enable/disable functionality for the import button since we only want it enabled while on the item page.
function ItemPageLeft()
	amazonForm.ImportButton.BarButton.Enabled = false;
	amazonForm.Browser:RegisterPageHandler("custom", "CheckItemPageLoaded", "ItemPageLoaded", false);
end

function ItemPageLoaded()
	amazonForm.ImportButton.BarButton.Enabled = true;
	amazonForm.Browser:RegisterPageHandler("custom", "CheckItemPageLeft", "ItemPageLeft", true);
end

function ImportPrice()
	local priceContainer = amazonForm.Browser:GetElementInFrame(nil, "rbbContainer");

	if priceContainer == nil then
		priceContainer = amazonForm.Browser:GetElementInFrame(nil, "buybox");
		if priceContainer == nil then
			priceContainer = amazonForm.Browser:GetElementInFrame(nil, "priceBlock");
			if priceContainer == nil then
				return false;
			end
		end
	end
	
	local priceElements = priceContainer:GetElementsByTagName("span");

	if priceElements ~= nil then
		for i=0, priceElements.Count - 1 do
			local element = amazonForm.Browser:GetElementByCollectionIndex(priceElements, i);
			if element ~= nil then
				if ClassNamesContain(element:GetAttribute("className"), "offer-price") or ClassNamesContain(element:GetAttribute("className"), "bb_price") then
					local priceText = element.InnerText;
					priceText = string.sub(priceText, 2);
					SetFieldValue("Item", GetSetting("PriceField"), priceText);
					ExecuteCommand("SwitchTab", {"Details"});
					return;
				end
			end
		end
	end
	
	priceElements = priceContainer:GetElementsByTagName("b");
	if priceElements == nil then
		return
	end
	for i=0, priceElements.Count - 1 do
		local element = amazonForm.Browser:GetElementByCollectionIndex(priceElements, i);
		if element ~= nil then
			if ClassNamesContain(element:GetAttribute("className"), "priceLarge") then
				local priceText = element.InnerText;
				priceText = string.sub(priceText, 2);
				SetFieldValue("Item", GetSetting("PriceField"), priceText);
				ExecuteCommand("SwitchTab", {"Details"});
				return;
			end
		end
	end

	return;
end

function ClassNamesContain(classesString, key)
	for token in string.gmatch(classesString, "[^%s]+") do
		if token:upper() == key:upper() then
			return true;
		end
	end
	return false;
end